//
//  CCViewController.m
//  WordLearning
//
//  Created by neoman on 5/22/13.
//  Copyright (c) 2013 org.codesme. All rights reserved.
//

#import "CCViewController.h"
#import "WordStore.h"
#import "WordView.h"
#import "Word.h"

static CGPoint currentCenter;

@implementation CCViewController

@synthesize currentWordView, nextWordView, deleteView, checkMarkView;

- (void)viewDidLoad
{
    UIPanGestureRecognizer *pan = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(changeWord:)];
    
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapWord:)];
    
    currentWordView.gestureRecognizers = @[pan, tap];
    
    [[WordStore sharedStore] fetchAllWordsWithCompletion:^{
        [self fetchCurrentWord];
    }];
}

- (void)fetchCurrentWord
{
    [[WordStore sharedStore]fetchAWordWithCompletion:^(Word *word) {
        self.currentWordView.word = word;
        [[WordStore sharedStore] fetchWordAudioWithName:word.nameEn];
        [self fetchNextWord];
    }];
}

- (void)fetchNextWord
{
    [[WordStore sharedStore]fetchAWordWithCompletion:^(Word *word) {
        self.nextWordView.word = word;
    }];
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
}


- (void)changeWord:(UIPanGestureRecognizer *)gesture
{
    CGPoint translation = [gesture translationInView:self.view];
    CGFloat dx = translation.x;
    
    UIView *deleteORcheckMark = dx < 0? checkMarkView : deleteView;
    UIView *deleteORcheckMarkOther = dx > 0? checkMarkView : deleteView;
    CGFloat deleteOrCheckMarkX = dx < 0? 295.0f : 25.0f;

    if (gesture.state == UIGestureRecognizerStateBegan) {
        currentCenter = currentWordView.center;
    }
    
    
    // total translation in x-axis
    CGFloat dxForDeleteOrCheckMark = 0.0f;
    if (ABS(dx) < 135.0f) {
        dxForDeleteOrCheckMark = dx * 85.0f / 135.0f;
        deleteORcheckMark.backgroundColor = [UIColor whiteColor];
    } else {
        dxForDeleteOrCheckMark = (dx < 0.0f) ? -85.0f + dx + 135.0f : 85.0f + dx - 135.0f;
        deleteORcheckMark.backgroundColor = (dx > 0) ? [UIColor colorWithRed:1.0f green:103.0f / 255.0f blue: 102.0f/255.0f alpha:1.0f] :[UIColor colorWithRed:151.0f/255.0f green:206.0f/255.0f blue:104.0f/255.0f alpha:1.0f];
    }
    
    currentWordView.center = CGPointMake(currentCenter.x + dx, currentCenter.y);
    deleteORcheckMark.hidden = NO;
    deleteORcheckMarkOther.hidden = YES;
    deleteORcheckMark.center = CGPointMake(deleteOrCheckMarkX + dxForDeleteOrCheckMark, 195.0f);
    
    if (gesture.state == UIGestureRecognizerStateEnded) {
        CGFloat desTx = 0.0f;
        CGFloat destXForDeleteOrCheck = 0.0F;
        NSNumber *whetherToGetNext = [NSNumber numberWithBool:NO];
        //CGFloat alpha = 1.0f;
        
        if (dx > 135.0f || dx < -135.0f) {
            desTx = dx > 0? currentCenter.x + 320.0f + 50 : currentCenter.x - 320.0f - 50.0f;
            destXForDeleteOrCheck = dx > 0? 25.0f + 320.0f : 295.0f - 320.0f;
            whetherToGetNext = [NSNumber numberWithBool:YES];
            if (dx < 0) {
                // swipe left means remembered that word
                [[WordStore sharedStore] rememberWord:currentWordView.word];
            }
            //alpha = 0.0f;
        }
        else {
            desTx = currentCenter.x;
            destXForDeleteOrCheck = dx > 0? 25.0f : 295.0f;
            whetherToGetNext = [NSNumber numberWithBool:NO];
        }
        
        [UIView beginAnimations:@"animation" context:(__bridge void*)whetherToGetNext];
        [UIView setAnimationDuration:0.3f];
        [UIView setAnimationCurve:UIViewAnimationCurveEaseIn];
        [UIView setAnimationDelegate:self];
        currentWordView.center = CGPointMake(desTx, currentCenter.y);
        deleteORcheckMark.center = CGPointMake(destXForDeleteOrCheck, 195.0f);
        //currentWordView.alpha = alpha;
        [UIView commitAnimations];

    }
}

 - (void)animationDidStop:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{
    BOOL whetherToGetNext = [(__bridge NSNumber *)context boolValue];
    
    if (whetherToGetNext) {        
        // set the currentView's gesture recognizers to nil
        NSArray *gestures = currentWordView.gestureRecognizers;
        currentWordView.gestureRecognizers = nil;
        
        nextWordView.gestureRecognizers = gestures;
        [self.view exchangeSubviewAtIndex:0 withSubviewAtIndex:3];
        
        currentWordView.center = nextWordView.center;
        currentWordView.alpha = 1.0f;
        [nextWordView setNeedsDisplay];
        currentWordView.showExplanation = NO;
        
        WordView *temp = currentWordView;
        currentWordView = nextWordView;
        nextWordView = temp;
        
        [[WordStore sharedStore] fetchWordAudioWithName:currentWordView.word.nameEn];
        [self fetchNextWord];
    }
    
    deleteView.backgroundColor = [UIColor whiteColor];
    deleteView.hidden = YES;
    checkMarkView.backgroundColor = [UIColor whiteColor];
    checkMarkView.hidden = YES;
}

- (void)tapWord:(UITapGestureRecognizer *)gesture;
{
    // set this property will update the view
    if (currentWordView.showExplanation == NO) {
        currentWordView.showExplanation = YES;
    }
    
    // get explanation audio from the server and play it
    [[WordStore sharedStore]fetchWordAudioWithName:currentWordView.word.nameCh];
}


@end
