//
//  WordStore.m
//  WordLearning
//
//  Created by neoman on 5/21/13.
//  Copyright (c) 2013 org.codesme. All rights reserved.
//

#import <AVFoundation/AVFoundation.h>

#import "WordStore.h"
#import "ConnectionActor.h"
#import "Word.h"

static BOOL hasSeeded = NO;

@interface WordStore ()
{
    NSArray *words;
    NSMutableDictionary *wordsExplanation;
    NSMutableDictionary *rememberedWords;
    AVAudioPlayer *play;
    int currentIndex;
}
@end

@implementation WordStore

+ (id)sharedStore
{
    // this class can be inherented
    static id sharedInstance = nil;
    
    if (sharedInstance == nil) {
        sharedInstance = [[super allocWithZone:nil] init];
    }
    
    return sharedInstance;
}


+ (id)allocWithZone:(NSZone *)zone
{
    return [self sharedStore];
}

- (id)init
{
    self = [super init];
    if (self) {        
        wordsExplanation = [NSKeyedUnarchiver unarchiveObjectWithFile:[[self fileDrectory] stringByAppendingPathComponent:@"explanations.archive"]];
        if (wordsExplanation == nil) {
            wordsExplanation = [[NSMutableDictionary alloc] init];
        }
        
        rememberedWords = [NSKeyedUnarchiver unarchiveObjectWithFile:[[self fileDrectory] stringByAppendingPathComponent:@"remembered.archive"]];
        
        if (rememberedWords == nil) {
            rememberedWords = [[NSMutableDictionary alloc] init];
        }
        
        //currentIndex = random() % words.count;
    }
    
    return self;
}

// archive all the words
- (BOOL)saveAllWords
{
    return [words writeToFile:[[self fileDrectory] stringByAppendingPathComponent:@"words.archive"] atomically:YES] && [NSKeyedArchiver archiveRootObject:wordsExplanation toFile:[[self fileDrectory] stringByAppendingPathComponent:@"explanations.archive"]] && [NSKeyedArchiver archiveRootObject:rememberedWords toFile:[[self fileDrectory] stringByAppendingPathComponent:@"remembered.archive"]];
}

// file path
- (NSString *)fileDrectory
{
    NSString *path = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    return path;
}

// init the request to fetch all the words from the server
- (void)fetchAllWordsWithCompletion: (void (^)(void)) completion
{
    words = [NSArray arrayWithContentsOfFile:[[self fileDrectory] stringByAppendingPathComponent:@"words.archive"]];
    if (words == nil) {
        NSURL *url = [NSURL URLWithString:@"http://suishoubei.com/static/js/courses/DEV.json"];
        NSURLRequest *request = [NSURLRequest requestWithURL:url];
        
        // use connection Actor to get the data from the request;
        ConnectionActor *conn = [[ConnectionActor alloc] init];
        conn.request = request;
        conn.completion = ^(NSMutableData *data){
            NSDictionary *json = [NSJSONSerialization JSONObjectWithData:data options:0 error:nil];
            
            NSString *wordsString = [json objectForKey:@"ens"];
            self->words = [wordsString componentsSeparatedByString:@"|"];
            self->currentIndex = random() % (self->words).count;
            completion();
        };
        
        [conn start];
    }
    else {
        completion();
    }
}

- (void)fetchAWordWithCompletion:(void (^)(Word *word)) acompletion
{
    if (hasSeeded == NO) {
        srandom(time(NULL));
        hasSeeded = YES;
    }
    int index = random() % words.count;
    
    while (index == currentIndex) {
        index = random() % words.count;
    }
    
    NSString *name = [words objectAtIndex:index];
    //NSString *name = @"siren";
    
    Word *word = [wordsExplanation objectForKey:name];
    
    if (word != nil) {
        acompletion(word);
        return;
    }
    
    // get the word explanation, and update the view when the explanation is recived.
    word  = [[Word alloc] init];
    NSURL *url = [NSURL URLWithString:[[NSString stringWithFormat:@"http://suishoubei.com/card?en=%@&card=1", name] stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
    NSURLRequest *request = [NSURLRequest requestWithURL:url];
    
    ConnectionActor *conn = [[ConnectionActor alloc] init];
    conn.request = request;
    conn.completion = ^(NSMutableData *data){
        NSDictionary *jsonWord = [NSJSONSerialization JSONObjectWithData:data options:0 error:nil];
        [word readFromJsonData:jsonWord];
        
        [self->wordsExplanation setObject:word forKey:word.nameEn];
        
        acompletion(word);
    };
    
    [conn start];
}

- (void)fetchWordAudioWithName:(NSString *)name
{
    // fetch the audio and let the completionblock decide with todo
    
    // first encode the name with base64
    NSString *nameBase64 = [self base64String:name];
    NSLog(@"%@", nameBase64);
    
    NSURL *url = [NSURL URLWithString:[NSString stringWithFormat:@"http://suishoubei.com/static/voices/%@.mp3", nameBase64]];
    NSURLRequest *request = [NSURLRequest requestWithURL:url];
    
    ConnectionActor *conn = [[ConnectionActor alloc] init];
    conn.request = request;
    conn.completion = ^(NSMutableData *data){
        NSError *error;
        
        play = [[AVAudioPlayer alloc] initWithData:data error:&error];
        play.numberOfLoops = 0;
        play.volume = 1.0f;
        [play prepareToPlay];
        
        if (play == nil)
            NSLog(@"%@", [error description]);
        else
            [play play];
    };
    
    [conn start];
}

- (void)rememberWord:(Word *)word
{
    [rememberedWords setObject:word forKey:word.nameEn];
}

- (NSString *)rememberHowMany
{
    return [NSString stringWithFormat:@"%d/%d", rememberedWords.count, words.count];
}

- (NSString *)base64String:(NSString *)str
{
    NSData *theData = [str dataUsingEncoding: NSUTF8StringEncoding];
    const uint8_t* input = (const uint8_t*)[theData bytes];
    NSInteger length = [theData length];
    
    static char table[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
    
    NSMutableData* data = [NSMutableData dataWithLength:((length + 2) / 3) * 4];
    uint8_t* output = (uint8_t*)data.mutableBytes;
    
    NSInteger i;
    for (i=0; i < length; i += 3) {
        NSInteger value = 0;
        NSInteger j;
        for (j = i; j < (i + 3); j++) {
            value <<= 8;
            
            if (j < length) {
                value |= (0xFF & input[j]);
            }
        }
        
        NSInteger theIndex = (i / 3) * 4;
        output[theIndex + 0] =                    table[(value >> 18) & 0x3F];
        output[theIndex + 1] =                    table[(value >> 12) & 0x3F];
        output[theIndex + 2] = (i + 1) < length ? table[(value >> 6)  & 0x3F] : '=';
        output[theIndex + 3] = (i + 2) < length ? table[(value >> 0)  & 0x3F] : '=';
    }
    
    return [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
}

@end
