//
//  WordView.m
//  WordLearning
//
//  Created by neoman on 5/22/13.
//  Copyright (c) 2013 org.codesme. All rights reserved.
//

#import "WordView.h"
#import "Word.h"

@interface WordView ()

@end

@implementation WordView

@synthesize word, showExplanation;

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        
    }
    return self;
}

- (void)drawRect:(CGRect)rect
{
    if (word == nil) {
        return;
    }
    
    CGContextRef context = UIGraphicsGetCurrentContext();
    
#define TITILEHEIGHT 24
    // TODO draw the word view's title views
    CGRect titleRect = CGRectMake(0.0f, 0.0f, 320.0f, TITILEHEIGHT);
    CGContextAddRect(context, titleRect);
    [[UIColor colorWithRed:160.0f/255.0f green:160.0f/255.0f blue:160.0f/255.0f alpha:0.4f]setFill];
    CGContextFillPath(context);
    
    
    UIFont *font = nil;
    NSString *string = nil;
    CGSize size;
    CGRect stringRect;
    
    // draw the statics
    string =[NSString stringWithFormat:@"IELTS[%@]", [self.word wordStatics]];
    font = [UIFont systemFontOfSize:12.0f];
    size = [string sizeWithFont:font];
    stringRect.size = size;
    stringRect.origin.x = 5.0f;
    stringRect.origin.y = (TITILEHEIGHT - size.height) / 2.0f;
    [[UIColor colorWithRed:160.0f/255.0f green:160.0f/255.0f blue:160.0f/255.0f alpha:1.0f]setFill];
    [string drawInRect:stringRect withFont:font];
    
    // draw the time
    NSTimeInterval interval = [[NSDate date] timeIntervalSinceDate: word.lastCheck];
    
    if (interval < 60) {
        string = @"new";
    }
    else if (interval < 3600)
        string = [NSString stringWithFormat: @"上次接触：%.0f分钟前",interval / 60];
    else if (interval < 60*60*24)
        string = [NSString stringWithFormat:@"上次接触：%.0f小时前", interval / 3600];
    else
        string = [NSString stringWithFormat:@"上次接触：%.0f天前", interval / 3600 /24];
    size = [string sizeWithFont:font];
    stringRect.size = size;
    stringRect.origin.x = 320.f - size.width - 5.0f;
    stringRect.origin.y = (TITILEHEIGHT - size.height) / 2.0f;
    [string drawInRect:stringRect withFont:font];
    
    // draw the word's pronetic
    string = [NSString stringWithFormat:@"/ %@ /", word.phoneticEn];
    if (word.phoneticEn != nil) {
        [[UIColor colorWithRed:53.0f/255.0f green:161.0f/255.0f blue:212.0f/255.0f alpha:1.0f]set];
        font = [UIFont systemFontOfSize:18.0f];
        string = [NSString stringWithFormat:@"/ %@ /", word.phoneticEn];
        size = [string sizeWithFont:font];
        stringRect.size = size;
        stringRect.origin.y = 75.0f;
        stringRect.origin.x = (320.0f - size.width) / 2.0f;
        [string drawInRect:stringRect withFont:font];
    }
    
    //draw the word's name
    [[UIColor blackColor]set];
    font = [UIFont systemFontOfSize:63.0f];
    string = word.nameEn;
    size = [string sizeWithFont:font];
    stringRect.size = size;
    stringRect.origin.y = 100.0f;
    stringRect.origin.x = (320.0f - size.width) / 2.0f;
    if (stringRect.origin.x < 0 ) {
        CGFloat pointPerChar = size.width / string.length;
        font = [UIFont systemFontOfSize: 300.0f/ string.length * 63.0f / pointPerChar];
        size = [string sizeWithFont:font];
        stringRect.size = size;
        stringRect.origin.y = 100.0f;
        stringRect.origin.x = (320.0f - size.width) / 2.0f;
    }
    [string drawInRect:stringRect withFont:font];
    
    if (showExplanation) {
        CGRect rectBox = CGRectMake(0.0f, 155.0f, 320.0f, 60.0f);
        font = [UIFont systemFontOfSize:20.0f];
        stringRect.size = [[word.nameCh stringByReplacingOccurrencesOfString:@"." withString:@""] sizeWithFont:font];
        if (stringRect.size.width > 280.0f) {
            stringRect.origin.x = 20.0f;
        } else{
            stringRect.origin.x = (rectBox.size.width - stringRect.size.width) / 2.0f;
        }
        stringRect.origin.y = rectBox.origin.y + (rectBox.size.height - stringRect.size.height) / 2.0f;
        
        // TODO draw the explanation
        [self drawExplanation:word.nameCh inRect:stringRect inContext:context];
        
        // TODO draw the sentences
        rectBox = CGRectMake(0.0f, 240.0f, 320.0f, 568.0f - 60.0f);
        [self drawSentenceInRect:rectBox inContext:context];
    }
    
}

- (void)setWord:(Word *)aword
{
    word = aword;
    [self setNeedsDisplay];
}

- (void)setShowExplanation:(BOOL)ashowExplanation
{
    showExplanation = ashowExplanation;
    [self setNeedsDisplay];
}

- (void)drawText:(NSString *)text withFont:(UIFont *)font withColor:(UIColor *)color atX:(CGFloat)x atY:(CGFloat)y
{
    [color setFill];
    CGSize size = [text sizeWithFont:font];
    CGRect rect = {.size = size};
    rect.origin.x = x;
    rect.origin.y = y;
    
    [text drawInRect:rect withFont:font];
}

- (void)drawExplanation:(NSString *)explanation inRect:(CGRect)rect inContext:(CGContextRef)context
{
    CGFloat startX = rect.origin.x;
    CGFloat startY = rect.origin.y;
    
    UIFont *formatFont = [UIFont systemFontOfSize:12.0f];
    CGRect formatRect = CGRectZero;
    UIColor *formartBackColor = [UIColor colorWithRed:136.0f/255.0f green:136.0f/255.0f blue:136.0f/255.0f alpha:1.0f];
    UIColor *formatFontColor = [UIColor whiteColor];
    
    UIFont *explaFont = [UIFont systemFontOfSize:20.0f];
    CGRect explaRect = CGRectZero;
    UIColor *explaFontColor = [UIColor blackColor];
    
    NSRegularExpression *regularEx1 = [NSRegularExpression regularExpressionWithPattern:@"([[a-z]+)\\.]*)([^[a-z]]+)" options:0 error:nil];
    
    NSRegularExpression *regularEx2 = [NSRegularExpression regularExpressionWithPattern:@"([a-z]+)\\." options:0 error:nil];
    
    NSArray *matches = [regularEx1 matchesInString:explanation options:0 range:NSMakeRange(0, explanation.length)];
    
    for (NSTextCheckingResult *match in matches) {
        NSString *format = [explanation substringWithRange:[match rangeAtIndex:1]];
        NSString *meaning = [explanation substringWithRange:[match rangeAtIndex:2]];
        
        explaRect.size = [meaning sizeWithFont:explaFont];
        
        NSArray *matches2 = [regularEx2 matchesInString:format options:0 range:NSMakeRange(0,format.length)];
        for (NSTextCheckingResult *match2 in matches2) {
            NSString *subFormat = [format substringWithRange:[match2 rangeAtIndex:1]];
            
            // draw the format : draw the background
            formatRect.size = [subFormat sizeWithFont:formatFont];
            if (startX + formatRect.size.width > 300.0f) {
                startX = 20.0f;
                startY = startY + explaRect.size.height;
            }
            formatRect.origin.x = startX;
            formatRect.origin.y = startY + (rect.size.height - formatRect.size.height) / 2.0f;
            
            CGRect backRect = formatRect;
            backRect.origin.x = backRect.origin.x - 5.0f;
            backRect.size.width = backRect.size.width + 10.0f;
            startX = startX + backRect.size.width + 3.0f;
            [formartBackColor setFill];
            UIBezierPath *bezierPath = [UIBezierPath bezierPathWithRoundedRect:backRect cornerRadius:5.0f];
            CGPathRef path = bezierPath.CGPath;
            CGContextAddPath(context, path);
            CGContextFillPath(context);
            
            // draw the format: draw the string
            [formatFontColor setFill];
            [subFormat drawInRect:formatRect withFont:formatFont];
            
          //  NSLog(@"%@", subFormat);
        }

        //draw the meaning
        [explaFontColor setFill];
        while (startX + explaRect.size.width > 300.0f) {
            CGFloat remainLength = 300.0f - startX;
            int remtainWords =(int)(remainLength / explaRect.size.width * meaning.length);
            explaRect.origin.x = startX;
            explaRect.size.width = remainLength;
            startX = 20.0f;
            explaRect.origin.y = startY +  (rect.size.height - explaRect.size.height) / 2.0f;
            startY = startY + explaRect.size.height;
            NSString *substring = [meaning substringToIndex:remtainWords];
            [substring drawInRect:explaRect withFont:explaFont];
            meaning = [meaning substringFromIndex:remtainWords];
            explaRect.size = [meaning sizeWithFont:explaFont];
        }
        explaRect.origin.x = startX;
        startX = startX +explaRect.size.width;
        explaRect.origin.y = startY;
        [meaning drawInRect:explaRect withFont:explaFont];
        //NSLog(@"%@", meaning);
    }
}

- (void)drawSentenceInRect:(CGRect)rect inContext:(CGContextRef)context
{
    CGFloat length[] = {5.0f, 5.0f};
    CGFloat dx = 20.0f;
    
    [[UIColor colorWithRed:192.0f/255.0f green:192.0f/255.0f blue:192.0f/255.0f alpha:1.0f] setStroke];
    CGContextSetLineDash(context, 0, length, 2);
    CGContextMoveToPoint(context, rect.origin.x + dx, rect.origin.y);
    CGContextAddLineToPoint(context, rect.origin.x+ rect.size.width - dx, rect.origin.y);
    CGContextStrokePath(context);
    
    // draw the sentence
    CGRect boxRect = CGRectInset(rect, dx, 25.0f);
    CGRect stringRect;
    
    UIFont *font = [UIFont systemFontOfSize:18.0f];
    stringRect.size = [word.sentenceEn sizeWithFont:font constrainedToSize:boxRect.size];
    stringRect.origin = boxRect.origin;
    
    [UIColor colorWithRed:96.0f/255.0f green:96.0f/255.0f blue:96.0f/255.0f alpha:1.0f];
    [word.sentenceEn drawInRect:stringRect withFont:font];
    
    stringRect.origin.y = stringRect.origin.y + stringRect.size.height + 10.0f;
    stringRect.size = [word.sentenceCh sizeWithFont:font constrainedToSize:boxRect.size];
    
    [word.sentenceCh drawInRect:stringRect withFont:font];
    
}

@end
